//
// COPYRIGHT LICENSE: This information contains sample code provided in source code form. You may copy, 
// modify, and distribute these sample programs in any form without payment to IBM for the purposes of 
// developing, using, marketing or distributing application programs conforming to the application 
// programming interface for the operating platform for which the sample code is written. 
// Notwithstanding anything to the contrary, IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS 
// AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED 
// WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, 
// TITLE, AND ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, 
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE 
// SAMPLE SOURCE CODE. IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS 
// OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.  
//
// (C) COPYRIGHT International Business Machines Corp., 2001,2011
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.pbw.jpa;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

/**
 * Customer is the bean mapping for the 
 * CUSTOMER table.  
 * 
 * @see Customer
 */
@Entity(name="Customer")
@Table(name="CUSTOMER", schema="APP")
@NamedQueries({
	@NamedQuery(
		name="removeAllCustomers",
		query="delete from Customer")
})
public class Customer
{
	@Id
	private String customerID;
	private String password;
	
	@NotNull @Size(min=1, message="First name must include at least one letter.")
	private String firstName;
	@NotNull @Size(min=1, message="Last name must include at least one letter.")
	private String lastName;
	@NotNull @Size(min=1, message="Address must include at least one letter.")
	private String addr1;
	private String addr2;
	@NotNull @Size(min=1, message="City name must include at least one letter.")
	private String addrCity;
	@NotNull @Size(min=2, message="State must include at least two letters.")
	private String addrState;
	@Pattern(regexp="\\d{5}", message="Zip code does not have 5 digits.")
	private String addrZip;
	@NotNull @Pattern(regexp="\\d{3}-\\d{3}-\\d{4}", message="Phone number does not match xxx-xxx-xxxx.")
	private String phone;
	
	public Customer() {}
	
	/** 
	 * Create a new Customer.
	 *
	 * @param key CustomerKey
	 * @param password Password used for this customer account.
	 * @param firstName First name of the customer.
	 * @param lastName Last name of the customer
	 * @param addr1 Street address of the customer
	 * @param addr2 Street address of the customer
	 * @param addrCity City 
	 * @param addrState State
	 * @param addrZip Zip code
	 * @param phone Phone number
	 */
	public Customer(String key, String password, 
			String firstName, String lastName,
			String addr1, String addr2,
			String addrCity, String addrState,
			String addrZip, String phone)
			{ 
		this.setCustomerID(key);
		this.setPassword(password);
		this.setFirstName(firstName);
		this.setLastName(lastName);
		this.setAddr1(addr1);
		this.setAddr2(addr2);
		this.setAddrCity(addrCity);
		this.setAddrState(addrState);
		this.setAddrZip(addrZip);
		this.setPhone(phone);
			}
	
	/**
	 * Verify password.
	 *
	 * @param password value to be checked.
	 * @return True, if password matches one stored.
	 */
	public boolean verifyPassword(String password)
	{
		return this.getPassword().equals(password);
	}
		
	/** Get the customer's full name.
	 * @return String of customer's full name. */
	public String getFullName() 
	{ 
		return this.getFirstName() + " " + this.getLastName(); 
	}
	
	public String getAddr1() {
		return addr1;
	}
	
	public void setAddr1(String addr1) {
		this.addr1 = addr1;
	}
	
	public String getAddr2() {
		return addr2;
	}
	
	public void setAddr2(String addr2) {
		this.addr2 = addr2;
	}
	
	public String getAddrCity() {
		return addrCity;
	}
	
	public void setAddrCity(String addrCity) {
		this.addrCity = addrCity;
	}
	
	public String getAddrState() {
		return addrState;
	}
	
	public void setAddrState(String addrState) {
		this.addrState = addrState;
	}
	
	public String getAddrZip() {
		return addrZip;
	}
	
	public void setAddrZip(String addrZip) {
		this.addrZip = addrZip;
	}
	
	public String getCustomerID() {
		return customerID;
	}
	
	public void setCustomerID(String customerID) {
		this.customerID = customerID;
	}
	
	public String getFirstName() {
		return firstName;
	}
	
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}
	
	public String getLastName() {
		return lastName;
	}
	
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}
	
	public String getPassword() {
		return password;
	}
	
	public void setPassword(String password) {
		this.password = password;
	}
	
	public String getPhone() {
		return phone;
	}
	
	public void setPhone(String phone) {
		this.phone = phone;
	}
		
}

